import chai from 'chai'
import React from 'react'
import ReactDOM from 'react-dom'
import sinon from 'sinon'
import ReactTestUtils from 'react-addons-test-utils'
import MaskedTextInput from '../src/MaskedTextInput.jsx'

const expect = chai.expect

describe('MaskedTextInput', () => {
  it('does not throw when instantiated', () => {
    expect(() => ReactTestUtils.renderIntoDocument(
      <MaskedTextInput pattern="111-111"/>
    )).not.to.throw()
  })

  it('renders a single input element', () => {
    const maskedTextInput = ReactTestUtils.renderIntoDocument(
      <MaskedTextInput pattern="111-111"/>
    )

    expect(
      () => ReactTestUtils.findRenderedDOMComponentWithTag(maskedTextInput, 'input')
    ).not.to.throw()
  })

  it('sets state correctly when initialized', () => {
    const maskedTextInput = ReactTestUtils.renderIntoDocument(
      <MaskedTextInput pattern="111-111"/>
    )

    expect(maskedTextInput.state).to.deep.equal({
      placeholder: '___-___',
      previousValue: '___-___',
      value: null,
      currentCaretPosition: null
    })
  })

  describe('input change', () => {
    it('updates the state when input changes', () => {
      const maskedTextInput = ReactTestUtils.renderIntoDocument(
        <MaskedTextInput pattern="111-111"/>
      )
      const input = ReactTestUtils.findRenderedDOMComponentWithTag(maskedTextInput, 'input')

      input.value = '2__-___'
      input.selectionStart = 1
      input.selectionEnd = 1

      ReactTestUtils.Simulate.change(input)

      expect(maskedTextInput.state).to.deep.equal({
        value: '2__-___',
        placeholder: '___-___',
        previousValue: '___-___',
        currentCaretPosition: 1
      })
    })

    it('calls user provided `onChange` if it exists', () => {
      const userOnChange = sinon.spy()
      const maskedTextInput = ReactTestUtils.renderIntoDocument(
        <MaskedTextInput pattern="111-111" onChange={userOnChange}/>
      )
      const input = ReactTestUtils.findRenderedDOMComponentWithTag(maskedTextInput, 'input')

      input.value = '2___-___'
      input.selectionStart = 1
      input.selectionEnd = 1

      ReactTestUtils.Simulate.change(input)

      expect(userOnChange.called).to.equal(true)
    })

    it('adjusts the position of the caret correctly when it updates', () => {
      const maskedTextInput = ReactTestUtils.renderIntoDocument(
        <MaskedTextInput pattern="(11)"/>
      )
      const input = ReactTestUtils.findRenderedDOMComponentWithTag(maskedTextInput, 'input')

      input.value = '(2_)'
      input.selectionStart = 1
      input.selectionEnd = 1

      maskedTextInput.refs.inputElement.focus()

      ReactTestUtils.Simulate.change(input)

      expect([
        maskedTextInput.refs.inputElement.selectionStart,
        maskedTextInput.refs.inputElement.selectionEnd
      ]).to.deep.equal([2,2])
    })

    it('does not attempt to update the position of the caret when the input is not focused', () => {
      const maskedTextInput = ReactTestUtils.renderIntoDocument(
        <MaskedTextInput pattern="(11)"/>
      )
      const input = ReactTestUtils.findRenderedDOMComponentWithTag(maskedTextInput, 'input')

      input.value = '(2_)'
      input.selectionStart = 1
      input.selectionEnd = 1

      ReactTestUtils.Simulate.change(input)

      expect([
        maskedTextInput.refs.inputElement.selectionStart,
        maskedTextInput.refs.inputElement.selectionEnd
      ]).to.deep.equal([1, 1])
    })
  })

  /*
  I don't know why this test is not working even though I've checked in the actual code that
  the value given to the input is `null`.
   */
  it.skip('never sets the value of the input to empty pattern', () => {
    const maskedTextInput = ReactTestUtils.renderIntoDocument(<MaskedTextInput pattern="(11)"/>)

    const input = ReactTestUtils.findRenderedDOMComponentWithTag(maskedTextInput, 'input')

    input.value = '(__)'

    ReactTestUtils.Simulate.change(input)

    console.log(maskedTextInput.refs.inputElement.value);

    expect(input.value).to.equal(null)
  })

  it('resets state `placeholder` and `value` when it receives props', () => {
    const node = document.createElement('div')
    const maskedTextInput = React.render(<MaskedTextInput pattern="(11)"/>, node)

    const input = ReactTestUtils.findRenderedDOMComponentWithTag(maskedTextInput, 'input')

    input.value = '(2_)'
    input.selectionStart = 1
    input.selectionEnd = 1

    ReactTestUtils.Simulate.change(input)

    React.render(<MaskedTextInput pattern="111"/>, node)

    expect(maskedTextInput.state).to.deep.equal({
      placeholder: '___',
      previousValue: '(__)',
      value: null,
      currentCaretPosition: 1
    })
  })
})
